# -*- coding: utf-8 -*-
"""
    pip_services3_container.build.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    The default container factory provides generic functionality on-demand, such as logging and performance monitoring.
    
    :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = ['DefaultContainerFactory']

from .DefaultContainerFactory import DefaultContainerFactory
